import { ProjectActionTypes } from './../projects/project.action.types';
import { CompanyActionUnion } from './companies.actions';
import { CompanyActionTypes } from './company.action.types';
import { Action } from '@ngrx/store';
import { MsgState } from '../msg.state';
import { CompanyState } from './company.state';
import { state } from '@angular/animations';
import { Company } from 'src/app/shared/models/company/company.model';

/**reducer fonction to handle actions
a reducer function need initial state and an action to handle*/

const companyInitialState: CompanyState = {
  currentCompany: null,
  companiesList: [],
  msgState: MsgState.INIT_STATE,
  serverMsgErron: 'no error msg',
};

export function companyReducer(
  companyState: CompanyState = companyInitialState,
  action: Action
) {
  switch (action.type) {
    // get all companies
    case CompanyActionTypes.GET_ALL_COMPANIES:
      return {
        ...companyState,
        msgState: MsgState.LOADING,
      };

    case CompanyActionTypes.GET_ALL_COMPANIES_SUCCESS:
      return {
        ...companyState,
        msgState: MsgState.LOADED,
        companiesList: (<CompanyActionUnion>action).payload,
      };

    case CompanyActionTypes.GET_ALL_COMPANIES_ERROR:
      return {
        ...companyState,
        msgState: MsgState.ERR,
        serverMsgError: (<CompanyActionUnion>action).payload,
      };

    //create form to update company
    case CompanyActionTypes.CREATE_COMPANY_FORM_UPDATE:
      return {
        ...companyState,
        msgState: MsgState.FORM_LOADING,
      };
    case CompanyActionTypes.CREATE_COMPANY_FORM_UPDATE_SUCCESS:
      let companyToUpdate: Company = (<CompanyActionUnion>action).payload;
      return {
        ...companyState,
        msgState: MsgState.FORM_LOADED,
        currentCompany: companyToUpdate,
      };

    case CompanyActionTypes.CREATE_COMPANY_FORM_UPDATE_ERROR:
      return {
        ...companyState,
        msgState: MsgState.FORM_LOAD_ERR,
        serverMsgError: (<CompanyActionUnion>action).payload,
      };

    //save form data to update company
    case CompanyActionTypes.UPDATE_COMPANY_WITH_FORM_DATA:
      return {
        ...companyState,
        msgState: MsgState.DEFAULT_MSG,
      };
    case CompanyActionTypes.UPDATE_COMPANY_WITH_FORM_DATA_SUCCESS:
      let updatedCompany: Company = (<CompanyActionUnion>action).payload;
      let companiesListCopy1: Company[] = [...companyState.companiesList];
      companiesListCopy1 = companiesListCopy1.map((company) =>
        company.id == updatedCompany.id ? updatedCompany : company
      );

      return{
        ...companyState,
        msgState: MsgState.UPDATED,
        companiesList: companiesListCopy1
      }

      case CompanyActionTypes.UPDATE_COMPANY_WITH_FORM_DATA_ERROR:
        return{
          ...companyState,
          msgState: MsgState.ERR,
          serverMsgError: (<CompanyActionUnion>action).payload
        };

        // delete a company
      case CompanyActionTypes.DELETE_COMPANY:
        return{
          ...companyState,
          msgState: MsgState.DELETE
        }

      case CompanyActionTypes.DELETE_COMPANY_SUCCESS:
        let deletedCompany: Company = (<CompanyActionUnion> action).payload;
        let deletedCompanyIndex = companyState.companiesList.indexOf(deletedCompany);
        let companiesListCopy2: Company[] = [...companyState.companiesList];
        companiesListCopy2 = companiesListCopy2.splice(deletedCompanyIndex);

        return{
          ...companyState,
          msgState: MsgState.DELETED,
          companiesList: companiesListCopy2
        };

      case CompanyActionTypes.DELETE_COMPANY_ERROR:
        return{
          ... companyState,
          msgState: MsgState.ERR,
          serverMsgError: (<CompanyActionUnion> action).payload
        };

    default:
      return { ...companyState };
  }
}
