import { Injectable } from '@angular/core';
import { CompanyActionTypes } from './company.action.types';
import { CompanyActionUnion, GetAllCompaniesActionSuccess, GetAllCompaniesActionError } from './companies.actions';
import { Observable, mergeMap, catchError, map, of } from 'rxjs';
import { CompaniesService } from './../../shared/services/companies.service';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { Company } from 'src/app/shared/models/company/company.model';

@Injectable()
export class CompanyGetSearchEffects {
  constructor(
    private companyActions: Actions,
    private companiesService: CompaniesService
  ) {}

  getAllCompaniesEffect : Observable<CompanyActionUnion> = createEffect(
    () => this.companyActions.pipe(
      ofType(CompanyActionTypes.GET_ALL_COMPANIES),
      mergeMap(()=>{
        return this.companiesService.getAllCompanies().pipe(
          map( (companies: Company[])=> new GetAllCompaniesActionSuccess(companies)),
          catchError(err=> of(new GetAllCompaniesActionError(err.message)))
        )
      })
    )
  );
}
