import { Router } from '@angular/router';
import { UsersService } from '../../shared/services/users.service';
import { Actions, createEffect, ofType } from '@ngrx/effects';
import { Injectable } from '@angular/core';
import { GetAllUsersActionError, GetAllUsersActionSuccess, GetUser_s_ProjectsError, GetUser_s_ProjectsSuccess, SearchUserActionError, SearchUserActionSuccess, UserActionUnion } from './users.actions';
import { Observable, mergeMap, map, catchError, of, tap } from 'rxjs';
import { User } from 'src/app/shared/models/user/user.model';
import { UserActionTypes } from './user.action.types';

@Injectable()
export class UserGetSearchEffects {
  constructor(
    private userActions: Actions,
    private usersService: UsersService,
    private router: Router
  ) {}

  //create getAllUsers effect

  getAllUsersEffect: Observable<UserActionUnion> = createEffect(() =>
    this.userActions.pipe(
      ofType(UserActionTypes.GET_ALL_USERS),
      mergeMap(() => {
        return this.usersService.getAllUsers().pipe(
          map((users: User[]) => new GetAllUsersActionSuccess(users)),
          catchError((err) => of(new GetAllUsersActionError(err.message)))
        );
      })
    )
  );

  //create searchUser effect

  searchUserEffect: Observable<UserActionUnion> = createEffect(() =>
    this.userActions.pipe(
      ofType(UserActionTypes.SEARCH_A_USER),
      mergeMap((userAction: UserActionUnion) => {
        return this.usersService.searchUser(userAction.payload).pipe(
          map((users: User[]) => new SearchUserActionSuccess(users)),
          catchError((err) => of(new SearchUserActionError(err.message)))
        );
      })
    )
  );

  // navigate to another component effect
  navigate:Observable<UserActionUnion> = createEffect(
    () => this.userActions.pipe(
        ofType(UserActionTypes.NAVIGATE_TO_USER_S_PROJECTS_COMPONENT_SUCCESS),
        tap(() => {
            this.router.navigate(['/user-s-projects']);
        })
    ),
    { dispatch: false }
);

  loadUserSProjectsEffect : Observable<UserActionUnion> = createEffect(
    ()=> this.userActions.pipe(
      ofType(UserActionTypes.GET_USER_S_PROJECTS),
      mergeMap( (action: UserActionUnion)=>{
        return this.usersService.getUser_sProjects().pipe(
          map( (users: any)=> new GetUser_s_ProjectsSuccess(users)),
          catchError(err => of(new GetUser_s_ProjectsError(err.message)))
        )
      })
    )
  );
}
